@@ -3,10 +3,19 @@
 # This file is part of rdiff-backup.
 #
 # rdiff-backup is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge MA
-# 02139, USA; either version 2 of the License, or (at your option) any
-# later version; incorporated herein by reference.
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.
+#
+# rdiff-backup is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with rdiff-backup; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+# USA
 
 """Hold a variety of constants usually set at initialization."""
 
@@ -14,7 +23,7 @@ import re, os
 
 
 # The current version of rdiff-backup
-version = "0.11.0"
+version = "0.11.1"
 
 # If this is set, use this value in seconds as the current time
 # instead of reading it from the clock.
@@ -46,10 +55,6 @@ change_ownership = None
 # permissions).
 change_mirror_perms = (process_uid != 0)
 
-# If true, temporarily change permissions of unreadable files in
-# the source directory to make sure we can read all files.
-change_source_perms = None
-
 # If true, try to reset the atimes of the source partition.
 preserve_atime = None
 
@@ -94,24 +99,9 @@ client_conn = None
 # list.
 changed_settings = []
 
-# rdiff-backup will try to checkpoint its state every
-# checkpoint_interval seconds.  Then when resuming, at most this
-# amount of time is lost.
-checkpoint_interval = 20
-
 # The RPath of the rdiff-backup-data directory.
 rbdir = None
 
-# Indicates if a resume or a lack of resume is forced.  This
-# should be None for the default.  0 means don't resume, and 1
-# means resume.
-resume = None
-
-# If there has been an aborted backup fewer than this many seconds
-# ago, attempt to resume it where it left off instead of starting
-# a new one.
-resume_window = 7200
-
 # This string is used when recognizing and creating time strings.
 # If the time_separator is ":", then W3 datetime strings like
 # 2001-12-07T04:22:01-07:00 are produced.  It can be set to "_" to
@@ -160,9 +150,9 @@ ssh_compression = 1
 # If true, print statistics after successful backup
 print_statistics = None
 
-# On the reader and writer connections, the following will be
-# replaced by the source and mirror Select objects respectively.
-select_source, select_mirror = None, None
+# On the writer connection, the following will be set to the mirror
+# Select iterator.
+select_mirror = None
 
 # On the backup writer connection, holds the root incrementing branch
 # object.  Access is provided to increment error counts.
@@ -256,23 +246,4 @@ def postset_regexp_local(name, re_string, flags):
 	if flags: globals()[name] = re.compile(re_string, flags)
 	else: globals()[name] = re.compile(re_string)
 
-def set_select(dsrpath, tuplelist, quote_mode, *filelists):
-	"""Initialize select object using tuplelist
 
-	Note that each list in filelists must each be passed as
-	separate arguments, so each is recognized as a file by the
-	connection.  Otherwise we will get an error because a list
-	containing files can't be pickled.
-
-	"""
-	global select_source, select_mirror
-	if dsrpath.source:
-		select_source = Select(dsrpath, quote_mode)
-		select_source.ParseArgs(tuplelist, filelists)
-	else:
-		select_mirror = Select(dsrpath, quote_mode)
-		select_mirror.ParseArgs(tuplelist, filelists)
-
-
-from rpath import * # kludge to avoid circularity - not needed in this module
-from selection import *
