@@ -1,26 +1,31 @@
+
 # Copyright 2002 Ben Escoto
 #
 # This file is part of rdiff-backup.
 #
 # rdiff-backup is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge MA
-# 02139, USA; either version 2 of the License, or (at your option) any
-# later version; incorporated herein by reference.
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.
+#
+# rdiff-backup is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with rdiff-backup; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+# USA
 
 """Start (and end) here - read arguments, set global settings, etc."""
 
-import getopt, sys, re
-from log import *
-from lazy import *
-from connection import *
-from rpath import *
-from destructive_stepping import *
-from robust import *
-from restore import *
-from highlevel import *
-from manage import *
-import Globals, Time, SetConnections
+from __future__ import generators
+import getopt, sys, re, os
+from log import Log, LoggerError
+import Globals, Time, SetConnections, selection, robust, rpath, \
+	   manage, backup, connection, restore, FilenameMapping, \
+	   Security, Hardlink
 
 
 action = None
@@ -38,23 +43,21 @@ def parse_cmdlineoptions(arglist):
 		try: return open(filename, "r")
 		except IOError: Log.FatalError("Error opening file %s" % filename)
 
-	try: optlist, args = getopt.getopt(arglist, "blmr:sv:V",
-		 ["backup-mode", "calculate-average",
-		  "change-source-perms", "chars-to-quote=",
-		  "checkpoint-interval=", "current-time=", "exclude=",
-		  "exclude-device-files", "exclude-filelist=",
-		  "exclude-filelist-stdin", "exclude-globbing-filelist",
-		  "exclude-mirror=", "exclude-other-filesystems",
-		  "exclude-regexp=", "exclude-special-files", "force",
-		  "include=", "include-filelist=", "include-filelist-stdin",
-		  "include-globbing-filelist", "include-regexp=",
-		  "list-increments", "mirror-only", "no-compression",
-		  "no-compression-regexp=", "no-hard-links", "no-resume",
-		  "null-separator", "parsable-output", "print-statistics",
-		  "quoting-char=", "remote-cmd=", "remote-schema=",
-		  "remove-older-than=", "restore-as-of=", "restrict=",
-		  "restrict-read-only=", "restrict-update-only=", "resume",
-		  "resume-window=", "server", "sleep-ratio=",
+	try: optlist, args = getopt.getopt(arglist, "blr:sv:V",
+		 ["backup-mode", "calculate-average", "chars-to-quote=",
+		  "current-time=", "exclude=", "exclude-device-files",
+		  "exclude-filelist=", "exclude-filelist-stdin",
+		  "exclude-globbing-filelist=", "exclude-mirror=",
+		  "exclude-other-filesystems", "exclude-regexp=",
+		  "exclude-special-files", "force", "include=",
+		  "include-filelist=", "include-filelist-stdin",
+		  "include-globbing-filelist=", "include-regexp=",
+		  "list-changed-since=", "list-increments", "no-compression",
+		  "no-compression-regexp=", "no-hard-links", "null-separator",
+		  "parsable-output", "print-statistics", "quoting-char=",
+		  "remote-cmd=", "remote-schema=", "remove-older-than=",
+		  "restore-as-of=", "restrict=", "restrict-read-only=",
+		  "restrict-update-only=", "server", "sleep-ratio=",
 		  "ssh-no-compression", "terminal-verbosity=", "test-server",
 		  "verbosity=", "version", "windows-mode",
 		  "windows-time-format"])
@@ -64,13 +67,9 @@ def parse_cmdlineoptions(arglist):
 	for opt, arg in optlist:
 		if opt == "-b" or opt == "--backup-mode": action = "backup"
 		elif opt == "--calculate-average": action = "calculate-average"
-		elif opt == "--change-source-perms":
-			Globals.set('change_source_perms', 1)
 		elif opt == "--chars-to-quote":
 			Globals.set('chars_to_quote', arg)
 			Globals.set('quoting_enabled', 1)
-		elif opt == "--checkpoint-interval":
-			Globals.set_integer('checkpoint_interval', arg)
 		elif opt == "--current-time":
 			Globals.set_integer('current_time', arg)
 		elif opt == "--exclude": select_opts.append((opt, arg))
@@ -101,18 +100,17 @@ def parse_cmdlineoptions(arglist):
 			select_opts.append((opt, arg))
 			select_files.append(sel_fl(arg))
 		elif opt == "--include-regexp": select_opts.append((opt, arg))
+		elif opt == "--list-changed-since":
+			restore_timestr, action = arg, "list-changed-since"
 		elif opt == "-l" or opt == "--list-increments":
 			action = "list-increments"
-		elif opt == "-m" or opt == "--mirror-only": action = "mirror"
 		elif opt == "--no-compression": Globals.set("compression", None)
 		elif opt == "--no-compression-regexp":
 			Globals.set("no_compression_regexp_string", arg)
 		elif opt == "--no-hard-links": Globals.set('preserve_hardlinks', 0)
-		elif opt == '--no-resume': Globals.resume = 0
 		elif opt == "--null-separator": Globals.set("null_separator", 1)
 		elif opt == "--parsable-output": Globals.set('parsable_output', 1)
-		elif opt == "--print-statistics":
-			Globals.set('print_statistics', 1)
+		elif opt == "--print-statistics": Globals.set('print_statistics', 1)
 		elif opt == "--quoting-char":
 			Globals.set('quoting_char', arg)
 			Globals.set('quoting_enabled', 1)
@@ -130,9 +128,6 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "--restrict-update-only":
 			Globals.security_level = "update-only"
 			Globals.restrict_path = arg
-		elif opt == '--resume': Globals.resume = 1
-		elif opt == '--resume-window':
-			Globals.set_integer('resume_window', arg)
 		elif opt == "-s" or opt == "--server":
 			action = "server"
 			Globals.server = 1
@@ -152,6 +147,7 @@ def parse_cmdlineoptions(arglist):
 			Globals.set('quoting_enabled', 1)
 			Globals.set('preserve_hardlinks', 0)
 			select_opts.append(("--exclude-special-files", None))
+			assert 0, "Windows mode doesn't work in this version!"
 		elif opt == '--windows-time-format':
 			Globals.set('time_separator', "_")
 		else: Log.FatalError("Unknown option %s" % opt)
@@ -164,7 +160,7 @@ def set_action():
 		if l == 0: commandline_error("No arguments given")
 		elif l == 1: action = "restore"
 		elif l == 2:
-			if RPath(Globals.local_connection, args[0]).isincfile():
+			if rpath.RPath(Globals.local_connection, args[0]).isincfile():
 				action = "restore"
 			else: action = "backup"
 		else: commandline_error("Too many arguments given")
@@ -173,11 +169,11 @@ def set_action():
 		commandline_error("No arguments given")
 	if l > 0 and action == "server":
 		commandline_error("Too many arguments given")
-	if l < 2 and (action == "backup" or action == "mirror" or
-				  action == "restore-as-of"):
+	if l < 2 and (action == "backup" or action == "restore-as-of"):
 		commandline_error("Two arguments are required (source, destination).")
 	if l == 2 and (action == "list-increments" or
-				   action == "remove-older-than"):
+				   action == "remove-older-than" or
+				   action == "list-changed-since"):
 		commandline_error("Only use one argument, "
 						  "the root of the backup directory")
 	if l > 2 and action != "calculate-average":
@@ -190,33 +186,25 @@ def commandline_error(message):
 
 def misc_setup(rps):
 	"""Set default change ownership flag, umask, relay regexps"""
-	if ((len(rps) == 2 and rps[1].conn.os.getuid() == 0) or
-		(len(rps) < 2 and os.getuid() == 0)):
-		# Allow change_ownership if destination connection is root
-		for conn in Globals.connections:
-			conn.Globals.set('change_ownership', 1)
-		for rp in rps: rp.setdata() # Update with userinfo
-
 	os.umask(077)
 	Time.setcurtime(Globals.current_time)
 	FilenameMapping.set_init_quote_vals()
 	SetConnections.UpdateGlobal("client_conn", Globals.local_connection)
-
-	# This is because I originally didn't think compiled regexps
-	# could be pickled, and so must be compiled on remote side.
 	Globals.postset_regexp('no_compression_regexp',
 						   Globals.no_compression_regexp_string)
-
-	for conn in Globals.connections: Robust.install_signal_handlers()
+	for conn in Globals.connections:
+		conn.robust.install_signal_handlers()
+		conn.Hardlink.initialize_dictionaries()
 
 def take_action(rps):
 	"""Do whatever action says"""
-	if action == "server": PipeConnection(sys.stdin, sys.stdout).Server()
+	if action == "server":
+		connection.PipeConnection(sys.stdin, sys.stdout).Server()
 	elif action == "backup": Backup(rps[0], rps[1])
-	elif action == "restore": restore(*rps)
+	elif action == "restore": Restore(*rps)
 	elif action == "restore-as-of": RestoreAsOf(rps[0], rps[1])
-	elif action == "mirror": Mirror(rps[0], rps[1])
 	elif action == "test-server": SetConnections.TestConnections()
+	elif action == "list-changed-since": ListChangedSince(rps[0])
 	elif action == "list-increments": ListIncrements(rps[0])
 	elif action == "remove-older-than": RemoveOlderThan(rps[0])
 	elif action == "calculate-average": CalculateAverage(rps)
@@ -240,44 +228,21 @@ def Main(arglist):
 	cleanup()
 
 
-def Mirror(src_rp, dest_rp):
-	"""Turn dest_path into a copy of src_path"""
-	Log("Mirroring %s to %s" % (src_rp.path, dest_rp.path), 5)
-	mirror_check_paths(src_rp, dest_rp)
-	# Since no "rdiff-backup-data" dir, use root of destination.
-	SetConnections.UpdateGlobal('rbdir', dest_rp)
-	SetConnections.BackupInitConnections(src_rp.conn, dest_rp.conn)
-	backup_init_select(src_rp, dest_rp)
-	HighLevel.Mirror(src_rp, dest_rp)
-
-def mirror_check_paths(rpin, rpout):
-	"""Check paths and return rpin, rpout"""
-	if not rpin.lstat():
-		Log.FatalError("Source directory %s does not exist" % rpin.path)
-	if rpout.lstat() and not force: Log.FatalError(
-"""Destination %s exists so continuing could mess it up.  Run
-rdiff-backup with the --force option if you want to mirror anyway.""" %
-		rpout.path)
-
-
 def Backup(rpin, rpout):
 	"""Backup, possibly incrementally, src_path to dest_path."""
 	SetConnections.BackupInitConnections(rpin.conn, rpout.conn)
-	backup_init_select(rpin, rpout)
+	backup_set_select(rpin)
 	backup_init_dirs(rpin, rpout)
-	RSI = Globals.backup_writer.Resume.ResumeCheck()
-	SaveState.init_filenames()
 	if prevtime:
 		Time.setprevtime(prevtime)
-		HighLevel.Mirror_and_increment(rpin, rpout, incdir, RSI)
-	else: HighLevel.Mirror(rpin, rpout, incdir, RSI)
+		backup.Mirror_and_increment(rpin, rpout, incdir)
+	else: backup.Mirror(rpin, rpout)
 	rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
 
-def backup_init_select(rpin, rpout):
-	"""Create Select objects on source and dest connections"""
-	rpin.conn.Globals.set_select(DSRPath(1, rpin), select_opts,
-								 None, *select_files)
-	rpout.conn.Globals.set_select(DSRPath(None, rpout), select_mirror_opts, 1)
+def backup_set_select(rpin):
+	"""Create Select objects on source connection"""
+	rpin.conn.backup.SourceStruct.set_source_select(rpin, select_opts,
+													*select_files)
 
 def backup_init_dirs(rpin, rpout):
 	"""Make sure rpin and rpout are valid, init data dir and logging"""
@@ -296,17 +261,21 @@ def backup_init_dirs(rpin, rpout):
 
 	datadir = rpout.append("rdiff-backup-data")
 	SetConnections.UpdateGlobal('rbdir', datadir)
-	incdir = RPath(rpout.conn, os.path.join(datadir.path, "increments"))
+	incdir = rpath.RPath(rpout.conn, os.path.join(datadir.path, "increments"))
 	prevtime = backup_get_mirrortime()
 
 	if rpout.lstat():
 		if rpout.isdir() and not rpout.listdir(): # rpout is empty dir
 			rpout.chmod(0700) # just make sure permissions aren't too lax
 		elif not datadir.lstat() and not force: Log.FatalError(
-"""Destination directory %s exists, but does not look like a
-rdiff-backup directory.  Running rdiff-backup like this could mess up
-what is currently in it.  If you want to update or overwrite it, run
-rdiff-backup with the --force option.""" % rpout.path)
+"""Destination directory
+
+%s
+
+exists, but does not look like a rdiff-backup directory.  Running
+rdiff-backup like this could mess up what is currently in it.  If you
+want to update or overwrite it, run rdiff-backup with the --force
+option.""" % rpout.path)
 
 	if not rpout.lstat():
 		try: rpout.mkdir()
@@ -348,8 +317,7 @@ def backup_get_mirrortime():
 """Warning: duplicate current_mirror files found.  Perhaps something
 went wrong during your last backup?  Using """ + mirrorrps[-1].path, 2)
 
-	timestr = mirrorrps[-1].getinctime()
-	return Time.stringtotime(timestr)
+	return mirrorrps[-1].getinctime()
 
 def backup_touch_curmirror_local(rpin, rpout):
 	"""Make a file like current_mirror.time.data to record time
@@ -359,14 +327,14 @@ def backup_touch_curmirror_local(rpin, rpout):
 
 	"""
 	datadir = Globals.rbdir
-	map(RPath.delete, backup_get_mirrorrps())
+	map(rpath.RPath.delete, backup_get_mirrorrps())
 	mirrorrp = datadir.append("current_mirror.%s.%s" % (Time.curtimestr,
 														"data"))
 	Log("Touching mirror marker %s" % mirrorrp.path, 6)
 	mirrorrp.touch()
-	RPath.copy_attribs(rpin, rpout)
+	rpath.copy_attribs(rpin, rpout)
 
-def restore(src_rp, dest_rp = None):
+def Restore(src_rp, dest_rp = None):
 	"""Main restoring function
 
 	Here src_rp should be an increment file, and if dest_rp is
@@ -374,8 +342,7 @@ def restore(src_rp, dest_rp = None):
 
 	"""
 	rpin, rpout = restore_check_paths(src_rp, dest_rp)
-	time = Time.stringtotime(rpin.getinctime())
-	restore_common(rpin, rpout, time)
+	restore_common(rpin, rpout, rpin.getinctime())
 
 def RestoreAsOf(rpin, target):
 	"""Secondary syntax for restore operation
@@ -391,12 +358,14 @@ def RestoreAsOf(rpin, target):
 
 def restore_common(rpin, target, time):
 	"""Restore operation common to Restore and RestoreAsOf"""
+	if target.conn.os.getuid() == 0:
+		SetConnections.UpdateGlobal('change_ownership', 1)
 	mirror_root, index = restore_get_root(rpin)
 	mirror = mirror_root.new_index(index)
 	inc_rpath = datadir.append_path('increments', index)
 	restore_init_select(mirror_root, target)
 	restore_start_log(rpin, target, time)
-	Restore.Restore(inc_rpath, mirror, target, time)
+	restore.Restore(mirror, inc_rpath, target, time)
 	Log("Restore ended", 4)
 
 def restore_start_log(rpin, target, time):
@@ -421,11 +390,11 @@ def restore_check_paths(rpin, rpout, restoreasof = None):
 Try restoring from an increment file (the filenames look like
 "foobar.2001-09-01T04:49:04-07:00.diff").""" % rpin.path)
 
-	if not rpout: rpout = RPath(Globals.local_connection,
-								rpin.getincbase_str())
-	if rpout.lstat():
+	if not rpout: rpout = rpath.RPath(Globals.local_connection,
+									  rpin.getincbase_str())
+	if rpout.lstat() and not force:
 		Log.FatalError("Restore target %s already exists, "
-					   "and will not be overwritten." % rpout.path)
+					   "specify --force to overwrite." % rpout.path)
 	return rpin, rpout
 
 def restore_init_select(rpin, rpout):
@@ -436,8 +405,10 @@ def restore_init_select(rpin, rpout):
 	the restore operation isn't.
 
 	"""
-	Globals.set_select(DSRPath(1, rpin), select_mirror_opts, None)
-	Globals.set_select(DSRPath(None, rpout), select_opts, None, *select_files)
+	restore._select_mirror = selection.Select(rpin)
+	restore._select_mirror.ParseArgs(select_mirror_opts, [])
+	restore._select_mirror.parse_rbdir_exclude()
+	restore._select_source = selection.Select(rpout)
 
 def restore_get_root(rpin):
 	"""Return (mirror root, index) and set the data dir
@@ -461,7 +432,7 @@ def restore_get_root(rpin):
 
 	i = len(pathcomps)
 	while i >= 2:
-		parent_dir = RPath(rpin.conn, "/".join(pathcomps[:i]))
+		parent_dir = rpath.RPath(rpin.conn, "/".join(pathcomps[:i]))
 		if (parent_dir.isdir() and
 			"rdiff-backup-data" in parent_dir.listdir()): break
 		i = i-1
@@ -486,15 +457,13 @@ def restore_get_root(rpin):
 def ListIncrements(rp):
 	"""Print out a summary of the increments and their times"""
 	mirror_root, index = restore_get_root(rp)
-	Globals.rbdir = datadir = \
-					mirror_root.append_path("rdiff-backup-data")
-	mirrorrp = mirror_root.new_index(index)
-	inc_rpath = datadir.append_path('increments', index)
-	incs = Restore.get_inclist(inc_rpath)
-	mirror_time = Restore.get_mirror_time()
+	mirror_rp = mirror_root.new_index(index)
+	inc_rpath = Globals.rbdir.append_path('increments', index)
+	incs = restore.get_inclist(inc_rpath)
+	mirror_time = restore.MirrorStruct.get_mirror_time()
 	if Globals.parsable_output:
-		print Manage.describe_incs_parsable(incs, mirror_time, mirrorrp)
-	else: print Manage.describe_incs_human(incs, mirror_time, mirrorrp)
+		print manage.describe_incs_parsable(incs, mirror_time, mirror_rp)
+	else: print manage.describe_incs_human(incs, mirror_time, mirror_rp)
 
 
 def CalculateAverage(rps):
@@ -517,8 +486,8 @@ def RemoveOlderThan(rootrp):
 	timep = Time.timetopretty(time)
 	Log("Deleting increment(s) before %s" % timep, 4)
 
-	times_in_secs = map(lambda inc: Time.stringtotime(inc.getinctime()),
-						Restore.get_inclist(datadir.append("increments")))
+	times_in_secs = [inc.getinctime() for inc in 
+					 restore.get_inclist(datadir.append("increments"))]
 	times_in_secs = filter(lambda t: t < time, times_in_secs)
 	if not times_in_secs:
 		Log.FatalError("No increments older than %s found" % timep)
@@ -533,5 +502,15 @@ def RemoveOlderThan(rootrp):
 	if len(times_in_secs) == 1:
 		Log("Deleting increment at time:\n" + inc_pretty_time, 3)
 	else: Log("Deleting increments at times:\n" + inc_pretty_time, 3)
-	Manage.delete_earlier_than(datadir, time)
+	manage.delete_earlier_than(datadir, time)
+
+
+def ListChangedSince(rp):
+	"""List all the files under rp that have changed since restoretime"""
+	try: rest_time = Time.genstrtotime(restore_timestr)
+	except Time.TimeException, exc: Log.FatalError(str(exc))
+	mirror_root, index = restore_get_root(rp)
+	mirror_rp = mirror_root.new_index(index)
+	inc_rp = mirror_rp.append_path("increments", index)
+	restore.ListChangedSince(mirror_rp, inc_rp, rest_time)
 
