@@ -3,15 +3,24 @@
 # This file is part of rdiff-backup.
 #
 # rdiff-backup is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge MA
-# 02139, USA; either version 2 of the License, or (at your option) any
-# later version; incorporated herein by reference.
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.
+#
+# rdiff-backup is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with rdiff-backup; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+# USA
 
 """Support code for remote execution and data transfer"""
 
 from __future__ import generators
-import types, os, tempfile, cPickle, shutil, traceback, pickle, socket
+import types, os, tempfile, cPickle, shutil, traceback, pickle, socket, sys
 
 
 class ConnectionError(Exception): pass
@@ -87,7 +96,6 @@ class LowLevelPipeConnection(Connection):
 	b - string
 	q - quit signal
 	t - TempFile
-	d - DSRPath
 	R - RPath
 	r - RORPath only
 	c - PipeConnection object
@@ -112,11 +120,11 @@ class LowLevelPipeConnection(Connection):
 		"""Put an object into the pipe (will send raw if string)"""
 		Log.conn("sending", obj, req_num)
 		if type(obj) is types.StringType: self._putbuf(obj, req_num)
-		elif isinstance(obj, Connection): self._putconn(obj, req_num)
-		elif isinstance(obj, TempFile): self._puttempfile(obj, req_num)
-		elif isinstance(obj, DSRPath): self._putdsrpath(obj, req_num)
-		elif isinstance(obj, RPath): self._putrpath(obj, req_num)
-		elif isinstance(obj, RORPath): self._putrorpath(obj, req_num)
+		elif isinstance(obj, connection.Connection):self._putconn(obj, req_num)
+		elif isinstance(obj, TempFile.TempFile):
+			self._puttempfile(obj, req_num)
+		elif isinstance(obj, rpath.RPath): self._putrpath(obj, req_num)
+		elif isinstance(obj, rpath.RORPath): self._putrorpath(obj, req_num)
 		elif ((hasattr(obj, "read") or hasattr(obj, "write"))
 			  and hasattr(obj, "close")): self._putfile(obj, req_num)
 		elif hasattr(obj, "next"): self._putiter(obj, req_num)
@@ -137,7 +145,7 @@ class LowLevelPipeConnection(Connection):
 
 	def _putiter(self, iterator, req_num):
 		"""Put an iterator through the pipe"""
-		self._write("i", str(VirtualFile.new(RORPIter.ToFile(iterator))),
+		self._write("i", str(VirtualFile.new(rorpiter.ToFile(iterator))),
 					req_num)
 
 	def _puttempfile(self, tempfile, req_num):
@@ -146,11 +154,6 @@ class LowLevelPipeConnection(Connection):
 				   tempfile.index, tempfile.data)
 		self._write("t", cPickle.dumps(tf_repr, 1), req_num)
 
-	def _putdsrpath(self, dsrpath, req_num):
-		"""Put DSRPath into pipe.  See _putrpath"""
-		dsrpath_repr = (dsrpath.conn.conn_number, dsrpath.getstatedict())
-		self._write("d", cPickle.dumps(dsrpath_repr, 1), req_num)
-
 	def _putrpath(self, rpath, req_num):
 		"""Put an rpath into the pipe
 
@@ -230,12 +233,11 @@ class LowLevelPipeConnection(Connection):
 		elif format_string == "b": result = data
 		elif format_string == "f": result = VirtualFile(self, int(data))
 		elif format_string == "i":
-			result = RORPIter.FromFile(BufferedRead(VirtualFile(self,
-																int(data))))
+			result = rorpiter.FromFile(iterfile.BufferedRead(
+				VirtualFile(self, int(data))))
 		elif format_string == "t": result = self._gettempfile(data)
 		elif format_string == "r": result = self._getrorpath(data)
 		elif format_string == "R": result = self._getrpath(data)
-		elif format_string == "d": result = self._getdsrpath(data)
 		else:
 			assert format_string == "c", header_string
 			result = Globals.connection_dict[int(data)]
@@ -245,27 +247,19 @@ class LowLevelPipeConnection(Connection):
 	def _getrorpath(self, raw_rorpath_buf):
 		"""Reconstruct RORPath object from raw data"""
 		index, data = cPickle.loads(raw_rorpath_buf)
-		return RORPath(index, data)
+		return rpath.RORPath(index, data)
 
 	def _gettempfile(self, raw_tf_buf):
 		"""Return TempFile object indicated by raw_tf_buf"""
 		conn_number, base, index, data = cPickle.loads(raw_tf_buf)
-		return TempFile(Globals.connection_dict[conn_number],
-						base, index, data)
+		return TempFile.TempFile(Globals.connection_dict[conn_number],
+								 base, index, data)
 
 	def _getrpath(self, raw_rpath_buf):
 		"""Return RPath object indicated by raw_rpath_buf"""
 		conn_number, base, index, data = cPickle.loads(raw_rpath_buf)
-		return RPath(Globals.connection_dict[conn_number], base, index, data)
-
-	def _getdsrpath(self, raw_dsrpath_buf):
-		"""Return DSRPath object indicated by buf"""
-		conn_number, state_dict = cPickle.loads(raw_dsrpath_buf)
-		empty_dsrp = DSRPath("bypass", Globals.local_connection, None)
-		empty_dsrp.__setstate__(state_dict)
-		empty_dsrp.conn = Globals.connection_dict[conn_number]
-		empty_dsrp.file = None
-		return empty_dsrp
+		return rpath.RPath(Globals.connection_dict[conn_number],
+						   base, index, data)
 
 	def _close(self):
 		"""Close the pipes associated with the connection"""
@@ -313,6 +307,7 @@ class PipeConnection(LowLevelPipeConnection):
 			try: req_num, object = self._get()
 			except ConnectionQuit:
 				self._put("quitting", self.get_new_req_num())
+				self._close()
 				return
 			if req_num == desired_req_num: return object
 			else:
@@ -528,23 +523,11 @@ class VirtualFile:
 
 # everything has to be available here for remote connection's use, but
 # put at bottom to reduce circularities.
-import Globals, Time, Rdiff, Hardlink, FilenameMapping, C, Security, Main
-from static import *
-from lazy import *
-from log import *
-from iterfile import *
-from connection import *
-from rpath import *
-from robust import *
-from rorpiter import *
-from destructive_stepping import *
-from selection import *
-from statistics import *
-from increment import *
-from restore import *
-from manage import *
-from highlevel import *
-
+import Globals, Time, Rdiff, Hardlink, FilenameMapping, C, Security, \
+	   Main, rorpiter, selection, increment, statistics, manage, lazy, \
+	   iterfile, rpath, robust, restore, manage, backup, connection, \
+	   TempFile, SetConnections, librsync
+from log import Log
 
 Globals.local_connection = LocalConnection()
 Globals.connections.append(Globals.local_connection)
